@@ -1,5 +1,5 @@
 //  This file is distributed as part of the bit-babbler package.
-//  Copyright 2004 - 2015,  Ron <ron@debian.org>
+//  Copyright 2004 - 2016,  Ron <ron@debian.org>
 
 #ifndef _BB_CONTROL_SOCKET_H
 #define _BB_CONTROL_SOCKET_H
@@ -12,6 +12,7 @@
 #if EM_PLATFORM_POSIX
  #include <sys/stat.h>
  #include <sys/file.h>
+ #include <fcntl.h>
 #endif
 
 
@@ -287,7 +288,8 @@ namespace BitB
 
                 Log<2>( "+ ControlSock::Connection( %d )\n", fd );
 
-                int ret = pthread_create( &m_connectionthread, NULL, connection_thread, this );
+                int ret = pthread_create( &m_connectionthread, GetDefaultThreadAttr(),
+                                          connection_thread, this );
                 if( ret )
                 {
                     Close();
@@ -433,7 +435,8 @@ namespace BitB
         void start_server_thread()
         { //{{{
 
-            m_serverthread_err = pthread_create( &m_serverthread, NULL, server_thread, this );
+            m_serverthread_err = pthread_create( &m_serverthread, GetDefaultThreadAttr(),
+                                                 server_thread, this );
             if( m_serverthread_err )
             {
                 pthread_mutex_destroy( &m_servermutex );
