@@ -4,7 +4,7 @@
 //! @ingroup JsonParsing
 //! @brief Parser for JSON formatted data.
 //
-//  Copyright 2013 - 2016,  Ron <ron@debian.org>
+//  Copyright 2013 - 2017,  Ron <ron@debian.org>
 //  This file is distributed as part of the bit-babbler package.
 //
 ////////////////////////////////////////////////////////////////////
@@ -35,7 +35,7 @@
 //#define BB_HASH_JSON_OBJECT_MEMBERS
 
 #ifdef BB_HASH_JSON_OBJECT_MEMBERS
- #include <tr1/unordered_map>
+ #include <bit-babbler/unordered_map.h>
 #else
  #include <map>
 #endif
@@ -1176,11 +1176,11 @@ namespace BitB
                 // Always use the C locale for strtod, since we always want the
                 // radix character to be '.' even in locales where the decimal
                 // point might be a comma instead.
-                #if EM_PLATFORM_MSW
-                static _locale_t    clocale = _create_locale(LC_ALL, "C");
-                #else
+               #if HAVE_NEWLOCALE
                 static locale_t     clocale = newlocale(LC_ALL_MASK, "C", NULL);
-                #endif
+               #elif HAVE__CREATE_LOCALE
+                static _locale_t    clocale = _create_locale(LC_ALL, "C");
+               #endif
 
                 const char  *b = data.c_str() + pos;
                 char        *e;
@@ -1189,11 +1189,18 @@ namespace BitB
                 // which RFC 7159 doesn't permit, but that's probably not a problem unless
                 // someone wants to use this code as a strict validator.  We could always
                 // check for the hex prefix if we want or need to be fussy about that.
-                #if EM_PLATFORM_MSW
-                m_value = _strtod_l( b, &e, clocale );
-                #else
+               #if HAVE_STRTOD_L
                 m_value = strtod_l( b, &e, clocale );
-                #endif
+               #elif HAVE__STRTOD_L
+                m_value = _strtod_l( b, &e, clocale );
+               #else
+                // Fall back to using strtod in the current locale.  There's not much
+                // else we can safely do here except advise the user (which is done
+                // by the configure test) that they'll need to run this using the "C"
+                // locale if their default locale treats decimal numbers differently
+                // to that.
+                m_value = strtod( b, &e );
+               #endif
 
                 if( b == e )
                     throw Error( "Invalid JSON, bad number conversion at position %zu in '%s'",
@@ -1619,7 +1626,7 @@ namespace BitB
 
             //! Container type for JSON Object members
            #ifdef BB_HASH_JSON_OBJECT_MEMBERS
-            typedef std::tr1::unordered_map< std::string, Data::Handle >   Members;
+            typedef was_tr1::unordered_map< std::string, Data::Handle >     Members;
            #else
             typedef std::map< std::string, Data::Handle >   Members;
            #endif
