@@ -1,5 +1,5 @@
 //  This file is distributed as part of the bit-babbler package.
-//  Copyright 2010 - 2016,  Ron <ron@debian.org>
+//  Copyright 2010 - 2017,  Ron <ron@debian.org>
 
 #ifndef _BB_SECRET_SOURCE_H
 #define _BB_SECRET_SOURCE_H
@@ -12,6 +12,10 @@
  #include <poll.h>
 #endif
 
+#if EM_PLATFORM_MAC
+ #include <fcntl.h>
+#endif
+
 #if HAVE_LIBUDEV
 // Prior to udev-150 their headers don't wrap this for C++, so do it here until
 // the buggy distro releases catch up.  FC12 and CentOS 6, I'm looking at you.
@@ -142,6 +146,7 @@ namespace BitB
             unsigned                enable_mask;
             unsigned                disable_polarity;
             unsigned                bitrate;
+            unsigned                chunksize;
             unsigned                latency;
             unsigned                fold;
             unsigned                group;
@@ -155,6 +160,7 @@ namespace BitB
                 : enable_mask( 0x0f )
                 , disable_polarity( 0x00 )
                 , bitrate( 0 )
+                , chunksize( 0 )
                 , latency( unsigned(-1) )
                 , fold( unsigned(-1) )
                 , group( 0 )
@@ -286,7 +292,8 @@ namespace BitB
             // transfer (which is then the maximum time we'll block waiting to
             // perform an orderly exit).
             unsigned    chunksize = std::max( maxpacket,
-                                    std::min( 65536u,
+                                    std::min( options.chunksize ? options.chunksize
+                                                                : 65536u,
                                               powof2_down(m_bitrate / 32
                                                                     / maxpacket
                                                                     * maxpacket) ) );
@@ -550,7 +557,7 @@ namespace BitB
             typedef unsigned                                ID;
             typedef uint32_t                                Mask;
             typedef RefPtr< Group >                         Handle;
-            typedef std::tr1::unordered_map< ID, Handle >   Map;
+            typedef was_tr1::unordered_map< ID, Handle >    Map;
 
 
             struct Options
@@ -764,8 +771,8 @@ namespace BitB
 
                 buf = new uint8_t[size];
 
-                int ret = pthread_create( &thread, NULL, Pool::source_thread, this );
-
+                int ret = pthread_create( &thread, GetDefaultThreadAttr(),
+                                          Pool::source_thread, this );
                 if( ret )
                 {
                     group->ReleaseMask( groupmask );
@@ -1199,6 +1206,8 @@ namespace BitB
                 pthread_cancel( (*i)->thread );
 
 
+           #if HAVE_ABI_FORCED_UNWIND
+
             Log<3>( "Pool: waiting for thread termination\n" );
             while( ! m_threads.empty() )
             {
@@ -1225,6 +1234,39 @@ namespace BitB
 
             pthread_mutex_unlock( &m_mutex );
 
+           #else
+
+            // This isn't entirely "safe", new threads could be created or new
+            // sources added while we are tearing down the existing set, but in
+            // the case where we don't have full stack unwinding when a thread
+            // is cancelled, there are other things which are much more likely
+            // to be in an awkward state than this.  We shouldn't be here in
+            // "normal" operation anyway, only if we are in the process of a
+            // controlled termination, so we can get away with crash diving our
+            // way out of things to some extent.
+            //
+            // The main thing here is not to try to take the mutex again once
+            // we unlock it, because a rudely cancelled thread might still be
+            // holding it if the ScopedMutex wasn't properly unwound.
+
+            ThreadList      t;
+            Source::List    s;
+
+            t.swap( m_threads );
+            s.swap( m_sources );
+
+            pthread_mutex_unlock( &m_mutex );
+
+            Log<3>( "Pool: waiting for thread termination\n" );
+            for( ThreadList::iterator i = t.begin(), e = t.end(); i != e; ++ i )
+                pthread_join( *i, NULL );
+
+            Log<3>( "Pool: waiting for source termination\n" );
+            for( Source::List::iterator i = s.begin(), e = s.end(); i != e; ++ i )
+                pthread_join( (*i)->thread, NULL );
+
+           #endif
+
             pthread_cond_destroy( &m_sinkcond );
             pthread_cond_destroy( &m_sourcecond );
             pthread_mutex_destroy( &m_mutex );
@@ -1291,7 +1333,45 @@ namespace BitB
 
                     lock.Unlock();
                     pthread_cancel( p );
+
+                   #ifdef __FreeBSD__
+                    //{{{
+                    // On FreeBSD 11 (where hotplug support was first added)
+                    // if the device was removed while we were in an active
+                    // call to libusb_bulk_transfer(), then that call may
+                    // deadlock (typically somewhere in libusb_handle_events)
+                    // and we will never be able to join the cancelled thread.
+                    //
+                    // So to avoid it also deadlocking us, we limit the amount
+                    // of time we'll wait for the join to succeed, and if that
+                    // elapses, we'll bark about it, then just leak the thread
+                    // and its associated resources, and move on.
+                    //
+                    // It's not ideal, but it's about the best we can do until
+                    // the FreeBSD side of things is fixed.
+
+                    Log<4>( "Pool::RemoveSource: cancelling thread for %s\n",
+                                                    d->VerboseStr().c_str() );
+
+                    timespec           wait_until;
+                    GetFutureTimespec( wait_until, 2000 );
+
+                    int ret = pthread_timedjoin_np( p, NULL, &wait_until );
+
+                    if( ret )
+                        LogErr<0>( ret,
+                                   "Pool::RemoveSource: failed to join thread for removed device %s\n",
+                                   d->VerboseStr().c_str() );
+                    else
+                        Log<4>( "Pool::RemoveSource: joined thread for %s\n",
+                                                    d->VerboseStr().c_str() );
+                    //}}}
+                   #else
+
                     pthread_join( p, NULL );
+
+                   #endif
+
                     return;
                 }
             }
@@ -1382,7 +1462,7 @@ namespace BitB
             WriteFD        *w = new WriteFD( this, fd, len, handler, user_data );
             pthread_t       p;
 
-            int ret = pthread_create( &p, NULL, writefd_thread, w );
+            int ret = pthread_create( &p, GetDefaultThreadAttr(), writefd_thread, w );
 
             if( ret )
             {
@@ -1400,7 +1480,21 @@ namespace BitB
         void FeedKernelEntropy( const std::string &dev = std::string() )
         { //{{{
 
-           #if EM_PLATFORM_LINUX
+           #if !EM_PLATFORM_LINUX
+
+            // This is normally defined in linux/random.h, but we'll reuse it
+            // here as a generic struct to share as much of the code below as
+            // we reasonably can between all platforms.
+            struct rand_pool_info {
+                    int         entropy_count;
+                    int         buf_size;
+                    uint32_t    buf[];
+            };
+
+           #endif
+
+
+           #if EM_PLATFORM_LINUX || EM_PLATFORM_MAC
 
             // Each time we wake up, either because the kernel pool has fallen
             // below its watermark or due to the timer expiring, we read a full
@@ -1476,6 +1570,9 @@ namespace BitB
                 rpi.entropy_count = n * 8;
                 rpi.buf_size = n;
 
+
+               #if EM_PLATFORM_LINUX
+
                 if( ioctl( fd, RNDADDENTROPY, &rpi ) )
                     throw SystemError( _("Pool::FeedKernelEntropy: ioctl failed") );
 
@@ -1484,6 +1581,24 @@ namespace BitB
 
                 if( r < 0 )
                     throw SystemError( _("Pool::FeedKernelEntropy: poll failed") );
+
+               #elif EM_PLATFORM_MAC
+
+                // MacOS has no method of signalling to us when its kernel might
+                // actually want more entropy, so we'll just feed it a new block
+                // each time the kernel_refill_time expires.
+                ssize_t r = write( fd, buf, n );
+
+                if( r < 0 )
+                    throw SystemError( _("Pool::FeedKernelEntropy: write to kernel device failed") );
+
+                usleep( timeout * 1000 );
+
+               #else
+                // We should only be here if we added extra platforms to the
+                // outer condition and didn't actually add them here too ...
+                #error "You forgot to implement this didn't you ..."
+               #endif
             }
 
            #else
@@ -1499,7 +1614,7 @@ namespace BitB
             ScopedMutex     lock( &m_mutex );
             pthread_t       p;
 
-            int ret = pthread_create( &p, NULL, feedkernel_thread, this );
+            int ret = pthread_create( &p, GetDefaultThreadAttr(), feedkernel_thread, this );
 
             if( ret )
                 throw SystemError( ret, _("Pool::FeedKernelEntropyAsync: "
@@ -2096,7 +2211,9 @@ namespace BitB
                 pthread_mutex_init( &m_action_mutex, NULL );
                 pthread_cond_init( &m_action_cond, NULL );
 
-                int ret = pthread_create( &m_actor_thread, NULL, actor_thread, this );
+                const pthread_attr_t    *attr = GetDefaultThreadAttr();
+
+                int ret = pthread_create( &m_actor_thread, attr, actor_thread, this );
                 if( ret )
                 {
                     pthread_cond_destroy( &m_action_cond );
@@ -2105,7 +2222,7 @@ namespace BitB
                     throw SystemError( ret, _("UDEVMonitor: failed to create actor thread") );
                 }
 
-                ret = pthread_create( &m_monitor_thread, NULL, monitor_thread, this );
+                ret = pthread_create( &m_monitor_thread, attr, monitor_thread, this );
                 if( ret )
                 {
                     pthread_cancel( m_actor_thread );
@@ -2234,8 +2351,31 @@ namespace BitB
             // until someone changes how the handles work in that code.
             m_callbackhandle = 0;
 
+            // FreeBSD 11 bumped LIBUSB_API_VERSION to 0x01000102, but didn't add
+            // the libusb_has_capability() function ...  so if we're here, but
+            // don't have it, let's assume it succeeded until some other platform
+            // explodes in flames to say otherwise.
+           #if HAVE_LIBUSB_HAS_CAPABILITY
             if( libusb_has_capability(LIBUSB_CAP_HAS_HOTPLUG) )
+           #endif
             {
+               #ifdef __FreeBSD__
+                // On FreeBSD 11, we're seeing a 4 second delay between when we
+                // register the hotplug callback, and when it actually gets called
+                // to enumerate any already available devices.  In "normal" use,
+                // that probably isn't a major problem, but in the case where we
+                // are only requested to scan for available devices, that means
+                // we won't have any to report before it returns (without also
+                // doing a semi-random sleep to wait for these events).  So until
+                // we know whatever is causing that is fixed, on FreeBSD let's
+                // enumerate them explictly first, and let AddDevice() deal with
+                // weeding out the duplicates if/when we get them.
+                //
+                // There does seem to be some internal issue there, since we are
+                // also seeing libusb_exit() block for 4 seconds at shutdown too.
+                EnumerateDevices( vendorid, productid );
+               #endif
+
                 ScopedCancelState   cancelstate;
 
                 int ret = libusb_hotplug_register_callback( GetContext(),
@@ -2251,8 +2391,9 @@ namespace BitB
                                                             &m_callbackhandle
                                                           );
                 if( ret )
-                    Log<0>( _("DeviceList( %04x:%04x ): failed to register hotplug callback\n"),
-                                                                        vendorid, productid );
+                    LogUSBError<0>( ret,
+                                    _("DeviceList( %04x:%04x ): failed to register hotplug callback"),
+                                    vendorid, productid );
                 else
                     return;
             }
@@ -2290,7 +2431,15 @@ namespace BitB
            #if LIBUSB_SINCE(0x01000102)
             // Only available since 1.0.16
 
+            // FreeBSD 11 bumped LIBUSB_API_VERSION to 0x01000102, but didn't add
+            // the libusb_has_capability() function ...  so if we're here, but
+            // don't have it, let's assume it succeeded until some other platform
+            // exploded in flames to say otherwise.
+            #if HAVE_LIBUSB_HAS_CAPABILITY
             return libusb_has_capability(LIBUSB_CAP_HAS_HOTPLUG) ? true : false;
+            #else
+            return true;
+            #endif
 
            #else
 
