@@ -1,5 +1,5 @@
 //  This file is distributed as part of the bit-babbler package.
-//  Copyright 2010 - 2016,  Ron <ron@debian.org>
+//  Copyright 2010 - 2017,  Ron <ron@debian.org>
 
 #ifndef _BB_USBCONTEXT_H
 #define _BB_USBCONTEXT_H
@@ -25,7 +25,7 @@
 #include <set>
 #include <list>
 #include <vector>
-#include <tr1/unordered_map>
+#include <bit-babbler/unordered_map.h>
 #include <unistd.h>
 
 
@@ -743,7 +743,7 @@ namespace BitB
             private:
 
                 typedef std::set< uint8_t >                             ClaimSet;
-                typedef std::tr1::unordered_map< uint8_t, uint8_t >     AltMap;
+                typedef was_tr1::unordered_map< uint8_t, uint8_t >      AltMap;
 
                 Device::Handle          m_device;
                 libusb_device_handle   *m_handle;
@@ -1156,6 +1156,14 @@ namespace BitB
                 // which fortunately was the same release where LIBUSBX_API_VERSION
                 // was added, so since Debian Wheezy still has 1.0.11, and could be
                 // running a backported kernel, check for that too.
+                //
+                // For extra bonus fun, RHEL/CentOS 6 backported that ioctl to their
+                // 2.6.32 kernel, but didn't pull in the scatter-gather patch that
+                // was committed to the mainline kernel along with it, or a recent
+                // enough libusb to actually use it (they shipped with 1.0.9), so we
+                // need to work around their mongrel kernel, to cater for the case of
+                // users installing a later libusb(x) version on those systems.
+                // Which is a thing people really do to use later software there.
                 //}}}
                 m_maxtransfer = 16384;
 
@@ -1190,11 +1198,18 @@ namespace BitB
                         Log<2>( _("Device %03u:%03u has capabilities 0x%02x\n"),
                                                     m_busnum, m_devnum, devcaps );
 
+                        #ifdef USBDEVFS_CAP_BULK_SCATTER_GATHER
                         // Any of these capabilities should be enough to allow
                         // safely relaxing the 16kB transfer limit on our side.
                         if( devcaps & (USBDEVFS_CAP_BULK_CONTINUATION |
                                        USBDEVFS_CAP_NO_PACKET_SIZE_LIM |
                                        USBDEVFS_CAP_BULK_SCATTER_GATHER) )
+                        #else
+                        // But RHEL 6 backported USBDEVFS_GET_CAPABILITIES
+                        // without pulling in the scatter-gather patch ...
+                        if( devcaps & (USBDEVFS_CAP_BULK_CONTINUATION |
+                                       USBDEVFS_CAP_NO_PACKET_SIZE_LIM) )
+                        #endif
                             m_maxtransfer = DEFAULT_MAX_TRANSFER_SIZE;
                     }
                     close( fd );
@@ -1638,9 +1653,16 @@ namespace BitB
 
             pthread_mutex_destroy( &m_device_mutex );
 
+            // Extra debug log checkpoints here, because on FreeBSD 11, the
+            // call to libusb_exit can take 4 seconds to complete, which can
+            // otherwise look like our code has become hung up somewhere.
+            Log<4>( "USBContext: waiting for libusb_exit\n" );
+
             ScopedCancelState   cancelstate;
             libusb_exit( m_usb );
 
+            Log<4>( "USBContext: libusb_exit completed\n" );
+
         } //}}}
 
 
@@ -1754,7 +1776,7 @@ namespace BitB
 
             size_t  n = m_devices.size();
 
-            printf( _P("Have %zu device:\n",
+            printf( P_("Have %zu device:\n",
                        "Have %zu devices:\n", n), n );
 
             if( n > 0 )
