@@ -1,5 +1,5 @@
 //  This file is distributed as part of the bit-babbler package.
-//  Copyright 2014 - 2015,  Ron <ron@debian.org>
+//  Copyright 2014 - 2017,  Ron <ron@debian.org>
 
 #include "private_setup.h"
 
@@ -404,8 +404,8 @@ private:
 
         using BitB::SystemError;
 
-        int ret = pthread_create( &m_threadid, NULL, test_thread, this );
-
+        int ret = pthread_create( &m_threadid, BitB::GetDefaultThreadAttr(),
+                                  test_thread, this );
         if( ret )
             throw SystemError( ret, _("Test %s failed to create thread"), m_id.c_str() );
 
@@ -505,6 +505,7 @@ static void usage()
     printf("      --latency=ms          Override the USB latency timer\n");
     printf("  -f, --fold=n              Set the amount of entropy folding\n");
     printf("      --enable=mask         Select a subset of the generators\n");
+    printf("      --limit-max-xfer      Limit the transfer chunk size to 16kB\n");
     printf("\n");
     printf("Report bugs to support@bitbabbler.org\n");
     printf("\n");
@@ -525,6 +526,7 @@ int main( int argc, char *argv[] )
         LATENCY_OPT,
         ENABLE_OPT,
         NOCOLOUR_OPT,
+        LIMIT_MAX_XFER,
         VERSION_OPT
     };
 
@@ -538,6 +540,7 @@ int main( int argc, char *argv[] )
         { "latency",        required_argument,  NULL,      LATENCY_OPT },
         { "fold",           required_argument,  NULL,      'f' },
         { "enable",         required_argument,  NULL,      ENABLE_OPT },
+        { "limit-max-xfer", no_argument,        NULL,      LIMIT_MAX_XFER },
         { "no-colour",      no_argument,        NULL,      NOCOLOUR_OPT },
         { "all-results",    no_argument,        NULL,      'A' },
         { "verbose",        no_argument,        NULL,      'v' },
@@ -644,6 +647,14 @@ int main( int argc, char *argv[] )
                 break;
             }
 
+            case LIMIT_MAX_XFER:
+                if( device_options.empty() )
+                    default_options.chunksize = 16384;
+                else
+                    device_options.back().chunksize = 16384;
+
+                break;
+
             case NOCOLOUR_OPT:
                 opt_testoptions.colour = false;
                 break;
